@@ -38,7 +38,7 @@ extern uint8_t osd_seeking;
 #define OSD_MIN_NFO_HEIGHT (160)
 
 /* prototypes in display.c */
-inline void stride_memcpy(void * dst, const void * src, int width, int height, int dstStride, int srcStride);
+void stride_memcpy(void * dst, const void * src, int width, int height, int dstStride, int srcStride);
 void rgb2argb (uint8_t *rgbabuffer, uint8_t *rgbbuffer, int width, int height);
 void rgb2abgr (uint8_t *rgbabuffer, uint8_t *rgbbuffer, int width, int height);
 
@@ -132,6 +132,16 @@ void close_x_dialog(Display *dpy);
 int handle_xdlg_event (Display *dpy, XEvent *event);
 #endif
 
+#ifdef XFIB
+int   x_fib_show (Display *dpy, Window parent, int x, int y);
+void  x_fib_close (Display *dpy);
+int   x_fib_handle_events (Display *dpy, XEvent *event);
+int   x_fib_status ();
+char *x_fib_filename ();
+int   x_fib_cfg_filter_callback (int (*cb)(const char*));
+int fib_filter_movie_filename (const char *name);
+#endif
+
 
 #endif
 
@@ -198,7 +208,7 @@ void position_imlib2 (int x, int y);
  * Max Osx - quartz
  */
 
-#if defined PLATFORM_OSX && (defined __i386 || defined __ppc)
+#if defined PLATFORM_OSX && (defined __i386 || defined __ppc__)
 
 # define SUP_MACOSX 1
 #else
