@@ -1,4 +1,7 @@
-/* xjadeo - jack video monitor
+/* xjadeo - SDL display variant
+ *
+ * (C) 2006-2014 Robin Gareus <robin@gareus.org>
+ * (C) 2006-2011 Luis Garrido <luisgarrido@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,32 +16,11 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
- *
- * (c) 2006 
- *  Robin Gareus <robin@gareus.org>
- *  Luis Garrido <luisgarrido@users.sourceforge.net>
- *
- * this file was inspired by playdv source code of http://libdv.sourceforge.net/.
- *  - (c) 2000 Charles 'Buck' Krasic 
- *  - (c) 2000 Erik Walthinsen 
- *
  */
 
 #include "xjadeo.h"
 #include "display.h"
 
-extern long ts_offset; 
-extern char    *smpte_offset;
-extern int 	force_redraw; // tell the main event loop that some cfg has changed
-extern int 	interaction_override; // disable some options.
-extern double framerate;
-
-void jackt_toggle();
-void jackt_rewind();
-
-void calc_letterbox(int src_w, int src_h, int out_w, int out_h, int *sca_w, int *sca_h);
-void resized_sdl ();
-
 /*******************************************************************************
  * SDL
  */
@@ -47,21 +29,26 @@ void resized_sdl ();
 
 #define MYSDLFLAGS (SDL_HWSURFACE | SDL_RESIZABLE | SDL_DOUBLEBUF)
 
-  SDL_Surface* sdl_screen;
-  SDL_Overlay *sdl_overlay;
-  SDL_Rect sdl_rect;
-  SDL_Rect sdl_dest_rect;
-  int sdl_pic_format= SDL_YV12_OVERLAY; // fourcc
+static SDL_Surface* sdl_screen;
+static SDL_Overlay *sdl_overlay;
+static SDL_Rect sdl_rect;
+static SDL_Rect sdl_dest_rect;
+static int sdl_pic_format= SDL_YV12_OVERLAY; // fourcc
 
-	int full_screen_width = 1024;
-	int full_screen_height = 768;
+static int full_screen_width = 1024;
+static int full_screen_height = 768;
+static int	sdl_ontop = 0;
+static int sdl_full_screen = 0;
+static SDL_Rect sdl_oldsize;
+
+static void calc_letterbox(int src_w, int src_h, int out_w, int out_h, int *sca_w, int *sca_h);
+static void resized_sdl ();
 
 void close_window_sdl(void) {
 	if(sdl_overlay) SDL_FreeYUVOverlay(sdl_overlay); 
 	SDL_Quit();
 }
 
-
 int open_window_sdl (void) {
 	const SDL_VideoInfo *video_info;
 	int video_bpp;
@@ -91,14 +78,8 @@ int open_window_sdl (void) {
 	sdl_screen = SDL_SetVideoMode(sdl_rect.w, sdl_rect.h, video_bpp,MYSDLFLAGS);
 	SDL_WM_SetCaption("xjadeo", "xjadeo");
 
-// FIXME: on linux the SDL_*_OVERLAY are defined as FOURCC numbers rather than beeing abstract 
-// so we could try other ffmpeg/lqt compatible 420P formats as I420 (0x30323449)
-	sdl_overlay = SDL_CreateYUVOverlay(movie_width, movie_height, 0x30323449, sdl_screen);
-	sdl_pic_format=0x30323449;
-	if(!sdl_overlay || (!sdl_overlay->hw_overlay)) {
-		sdl_overlay = SDL_CreateYUVOverlay(movie_width, movie_height, SDL_YV12_OVERLAY, sdl_screen);
-		sdl_pic_format=SDL_YV12_OVERLAY;
-	}
+	newsrc_sdl();
+
 	if((!sdl_overlay)) 
 		fprintf(stderr, "NO OVERLAY\n");
 	if((!sdl_overlay || SDL_LockYUVOverlay(sdl_overlay)<0)) {
@@ -113,17 +94,13 @@ int open_window_sdl (void) {
 		fprintf(stderr,"unsupported SDL YV12.\n"); 
 		goto no_overlay;
 	}  
-#if 0  // verify YUV alignment
-	if ((sdl_overlay->pixels[1] - sdl_overlay->pixels[0]) != ( movie_width * movie_height) ||
-			(sdl_overlay->pixels[2] - sdl_overlay->pixels[1]) != ( movie_width * movie_height /4) ) {
-		printf("unsupported SDL YV12 pixel buffer alignment.\n"); 
-		goto no_overlay;
-	}  
-#endif
-	/*
-	SDL_Surface* icon = SDL_LoadBMP(iconName));
-	SDL_WM_SetIcon(icon, NULL);
-	*/
+
+	if (start_ontop) {
+		sdl_set_ontop(1);
+	}
+	if (start_fullscreen) {
+		sdl_toggle_fullscreen(1);
+	}
 
 	return(0);
  
@@ -134,13 +111,12 @@ no_sdl:
 	return 1;
 }
 
-void black_border_sdl(SDL_Rect b) {
-	//printf(" bb: +%i+%i %i %i\n", b.x, b.y, b.w, b.h);
+static void black_border_sdl(SDL_Rect b) {
 	SDL_FillRect(sdl_screen, &b, SDL_MapRGB(sdl_screen->format, 0,0,0));
 	SDL_UpdateRect(sdl_screen, b.x, b.y, b.w, b.h);
 }
 
-void resized_sdl () {
+static void resized_sdl () {
 	if (!want_letterbox) {
 		memcpy(&sdl_dest_rect, &sdl_rect, sizeof (SDL_Rect));
 		return;
@@ -152,10 +128,7 @@ void resized_sdl () {
 	sdl_dest_rect.h = dh;
 	sdl_dest_rect.x = (sdl_rect.w - sdl_dest_rect.w)/2;
 	sdl_dest_rect.y = (sdl_rect.h - sdl_dest_rect.h)/2;
-#if 0
-	SDL_FillRect(sdl_screen, &sdl_rect, SDL_MapRGB(sdl_screen->format, 0,0,0));
-	SDL_UpdateRect(sdl_screen, sdl_rect.x, sdl_rect.y, sdl_rect.w, sdl_rect.h);
-#else
+
 	SDL_Rect b;
 	if (sdl_dest_rect.y >0 ){
 		b.x=0;b.y=0; b.w=sdl_rect.w; b.h=sdl_dest_rect.y;
@@ -172,14 +145,12 @@ void resized_sdl () {
 		b.x=sdl_rect.w - sdl_dest_rect.x;b.y=0; b.w=sdl_dest_rect.x; b.h=sdl_rect.h;
 		black_border_sdl(b);
 	}
-#endif
 }
 
 void mousecursor_sdl(int action) {
-  static int sdl_mouse = 1;
-	if (action==2) sdl_mouse^=1;
-	else sdl_mouse=action?1:0;
-	SDL_ShowCursor(sdl_mouse);
+	if (action==2) hide_mouse ^= 1;
+	else hide_mouse = action ? 1 : 0;
+	SDL_ShowCursor(!hide_mouse);
 }
 
 void resize_sdl (unsigned int x, unsigned int y) { 
@@ -195,60 +166,127 @@ void getsize_sdl (unsigned int *x, unsigned int *y) {
 	if(y) *y = sdl_rect.h;
 }
 
-void get_window_pos_sdl (unsigned int *x, unsigned int *y) {
+void get_window_pos_sdl (int *rx, int *ry) {
 	SDL_SysWMinfo info;
 	SDL_VERSION(&info.version);
-	if (!( SDL_GetWMInfo(&info) > 0 )) { return; }
-#ifdef HAVE_WINDOWS
+	if ( SDL_GetWMInfo(&info) > 0 ) {
+#ifdef PLATFORM_WINDOWS
+
+#if 0 // legacy
 	WINDOWINFO w;
 	GetWindowInfo(info.window, &w);
 	*x= w.rcWindow.left;
 	*y= w.rcWindow.top;
-	//*w= w.rcWindow.right - w.rcWindow.left;
-	//*h= w.rcWindow.bottom - w.rcWindow.top;
 	printf("%ld - %ld\n", w.rcWindow.left, w.rcWindow.top);
 #endif
-#if (defined HAVE_LIBXV || defined HAVE_IMLIB || defined HAVE_IMLIB2)
-	if ( info.subsystem == SDL_SYSWM_X11 ) {
-		Window	dummy;
-		int xx,xy;
-		info.info.x11.lock_func();
-		XTranslateCoordinates(info.info.x11.display, info.info.x11.wmwindow, info.info.x11.fswindow, 0, 0, &xx, &xy, &dummy);
-		// TODO recurse until dummy!=None
-		info.info.x11.unlock_func();
-		*x= xx;
-		*y= xy;
-	}
+		RECT rect;
+		//GetWindowRect() <> SetWindowPos()
+		if (GetClientRect(info.window, &rect)) {
+			*rx = rect.left;
+			*ry = rect.top;
+			return;
+		}
+#elif (defined HAVE_LIBXV || defined HAVE_IMLIB2)
+		if (info.subsystem == SDL_SYSWM_X11 ) {
+			// NB. with SDL window decorations are not taken into account :(
+			Window	dummy;
+			info.info.x11.lock_func();
+			XTranslateCoordinates(info.info.x11.display, info.info.x11.wmwindow, DefaultRootWindow(info.info.x11.display), 0, 0, rx, ry, &dummy);
+			while (dummy !=None) {
+				int x = 0;
+				int y = 0;
+				XTranslateCoordinates(info.info.x11.display, info.info.x11.wmwindow, dummy, 0, 0, &x, &y, &dummy);
+				if (dummy!=None) {
+					(*rx)-=x; (*ry)-=y;
+				} else {
+					(*rx)+=x; (*ry)+=y;
+				}
+			}
+			info.info.x11.unlock_func();
+			return;
+		}
 #endif
+	}
+	if(rx) *rx=1;
+	if(ry) *ry=1;
 }
 
 void position_sdl(int x, int y) {
 	SDL_SysWMinfo info;
 	SDL_VERSION(&info.version);
 	if ( SDL_GetWMInfo(&info) > 0 ) {
-#ifdef HAVE_WINDOWS
-	SetWindowPos(info.window, NULL, x, y, sdl_rect.w, sdl_rect.h, 0);
+#ifdef PLATFORM_WINDOWS
+	SetWindowPos(info.window,
+			sdl_ontop ? HWND_TOPMOST : HWND_NOTOPMOST,
+			x, y, sdl_rect.w, sdl_rect.h, 0);
 #endif
-#if (defined HAVE_LIBXV || defined HAVE_IMLIB || defined HAVE_IMLIB2)
+#if (defined HAVE_LIBXV || defined HAVE_IMLIB2)
 	if ( info.subsystem == SDL_SYSWM_X11 ) {
 			info.info.x11.lock_func();
-#if 0 /* get root window size  -> center window  */
-			int x, y;
-			int w, h;
-			w = DisplayWidth(info.info.x11.display,
-			DefaultScreen(info.info.x11.display));
-
-			h = DisplayHeight(info.info.x11.display,
-			DefaultScreen(info.info.x11.display));
-			x = (w - screen->w)/2;
-			y = (h - screen->h)/2;
-#endif
 			XMoveWindow(info.info.x11.display, info.info.x11.wmwindow, x, y);
 			info.info.x11.unlock_func();
 		} 
 #endif
 	} 
-} 
+}
+
+#if (defined HAVE_LIBXV || defined HAVE_IMLIB2)
+static void net_wm_set_property(char *atom, int state) {
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+	if ( SDL_GetWMInfo(&info) <= 0 ) {
+		return;
+	}
+
+	XEvent xev;
+	int set = _NET_WM_STATE_ADD;
+	Atom type, property;
+
+	if (state == _NET_WM_STATE_TOGGLE) set = _NET_WM_STATE_TOGGLE;
+	else if (!state) set = _NET_WM_STATE_REMOVE;
+
+	type = XInternAtom(info.info.x11.display, "_NET_WM_STATE", True);
+	if (type == None) return;
+	property = XInternAtom(info.info.x11.display, atom, 0);
+	if (property == None) return;
+
+	xev.type = ClientMessage;
+	xev.xclient.type = ClientMessage;
+	xev.xclient.window = info.info.x11.wmwindow;
+	xev.xclient.message_type = type;
+	xev.xclient.format = 32;
+	xev.xclient.data.l[0] = set;
+	xev.xclient.data.l[1] = property;
+	xev.xclient.data.l[2] = 0;
+
+	if (!XSendEvent(info.info.x11.display, DefaultRootWindow(info.info.x11.display), False,
+				SubstructureRedirectMask|SubstructureNotifyMask, &xev))
+	{
+			fprintf(stderr,"error changing X11 property\n");
+	}
+}
+#endif
+
+void sdl_set_ontop (int action) {
+	if (action==2) sdl_ontop^=1;
+	else sdl_ontop=action;
+
+#ifdef PLATFORM_WINDOWS
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+	if ( SDL_GetWMInfo(&info) > 0 ) {
+		SetWindowPos(info.window,
+				sdl_ontop ? HWND_TOPMOST : HWND_NOTOPMOST,
+				0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+	}
+#elif (defined HAVE_LIBXV || defined HAVE_IMLIB2)
+	net_wm_set_property("_NET_WM_STATE_ABOVE", action);
+#endif
+}
+
+int sdl_get_ontop () {
+	return sdl_ontop;
+}
 
 void render_sdl (uint8_t *mybuffer) {
 	/* http://www.fourcc.org/indexyuv.htm */
@@ -278,9 +316,6 @@ void render_sdl (uint8_t *mybuffer) {
 	SDL_LockYUVOverlay(sdl_overlay);
 }
 
-int sdl_full_screen =0;
-SDL_Rect sdl_oldsize;
-
 int sdl_get_fullscreen () {
 	return (sdl_full_screen);
 }
@@ -300,9 +335,10 @@ void sdl_toggle_fullscreen(int action) {
     sdl_full_screen=1;
   }
 	resized_sdl();
+	force_redraw=1;
 }
 
-void calc_letterbox(int src_w, int src_h, int out_w, int out_h, int *sca_w, int *sca_h) {
+static void calc_letterbox(int src_w, int src_h, int out_w, int out_h, int *sca_w, int *sca_h) {
 	const float asp_src = movie_aspect?movie_aspect:(float)src_w/src_h;
   if (asp_src * out_h > out_w) {
     (*sca_w)=out_w;
@@ -313,9 +349,21 @@ void calc_letterbox(int src_w, int src_h, int out_w, int out_h, int *sca_w, int
 	}
 }
 
+void sdl_letterbox_change(void) {
+	resized_sdl();
+	force_redraw=1;
+}
+
 void newsrc_sdl (void) {
-	if(sdl_overlay) SDL_FreeYUVOverlay(sdl_overlay); 
-	sdl_overlay = SDL_CreateYUVOverlay(movie_width, movie_height, SDL_YV12_OVERLAY, sdl_screen);
+	if(sdl_overlay) SDL_FreeYUVOverlay(sdl_overlay);
+// FIXME: on linux the SDL_*_OVERLAY are defined as FOURCC numbers rather than beeing abstract
+// so we could try other ffmpeg/lqt compatible 420P formats as I420 (0x30323449)
+	sdl_overlay = SDL_CreateYUVOverlay(movie_width, movie_height, 0x30323449, sdl_screen);
+	sdl_pic_format=0x30323449;
+	if(!sdl_overlay || (!sdl_overlay->hw_overlay)) {
+		sdl_overlay = SDL_CreateYUVOverlay(movie_width, movie_height, SDL_YV12_OVERLAY, sdl_screen);
+		sdl_pic_format=SDL_YV12_OVERLAY;
+	}
 }
 
 void handle_X_events_sdl (void) {
@@ -325,7 +373,6 @@ void handle_X_events_sdl (void) {
 		switch(ev.type){
 			case SDL_VIDEOEXPOSE: // SDL render event
 				render_sdl(buffer);
-			//	printf("SDL render event.\n");
 				break;
 			case SDL_QUIT:
 				if ((interaction_override&OVR_QUIT_KEY) == 0) loop_flag=0;
@@ -338,118 +385,50 @@ void handle_X_events_sdl (void) {
 					} else {
 						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
 					}
-				} else if(ev.key.keysym.sym==SDLK_q) {
-					if ((interaction_override&OVR_QUIT_KEY) == 0) {
-						loop_flag=0;
-					} else {
-						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
-					}
 				} else if(ev.key.keysym.sym==SDLK_s) {
-					OSD_mode^=OSD_SMPTE;
-					force_redraw=1;
+					ui_osd_tc();
 				} else if(ev.key.keysym.sym==SDLK_a) {
-					// TODO always on top
+					sdl_set_ontop(sdl_ontop^=1);
 				} else if(ev.key.keysym.sym==SDLK_f) {
 					sdl_toggle_fullscreen(2);
 				} else if(ev.key.keysym.sym==SDLK_l) {
 						want_letterbox=!want_letterbox; 
-						resized_sdl();
-						force_redraw=1;
+						sdl_letterbox_change();
 				} else if(ev.key.keysym.sym==SDLK_m) { 
 					mousecursor_sdl(2);
-				} else if(ev.key.keysym.sym==SDLK_s) {
-					OSD_mode^=OSD_SMPTE;
-					force_redraw=1;
 				} else if(ev.key.keysym.sym==SDLK_v) {
-					OSD_mode^=OSD_FRAME; 
-					force_redraw=1;
+					ui_osd_fn();
 				} else if(ev.key.keysym.sym==SDLK_b) {
-						OSD_mode^=OSD_BOX;
-						force_redraw=1;
+					ui_osd_box();
+				} else if(ev.key.keysym.sym==SDLK_i) {
+					ui_osd_fileinfo();
+				} else if(ev.key.keysym.sym==SDLK_g) {
+					ui_osd_geo();
 				} else if(ev.key.keysym.sym== SDLK_c && ev.key.keysym.mod&KMOD_SHIFT) {
-					OSD_mode=0; 
-					force_redraw=1;
+					ui_osd_clear();
 				} else if(ev.key.keysym.sym== SDLK_LESS || (ev.key.keysym.sym== SDLK_COMMA && ev.key.keysym.mod&KMOD_SHIFT) ) { // '<'
-					unsigned int my_Width,my_Height;
-					getsize_sdl(&my_Width,&my_Height);
-					float step=0.2*my_Height;
-					my_Width-=floor(step*movie_aspect);
-					my_Height-=step;
-					resize_sdl(my_Width, my_Height);
+					XCresize_scale(-1);
 				} else if(ev.key.keysym.sym== SDLK_GREATER || (ev.key.keysym.sym== SDLK_PERIOD && ev.key.keysym.mod&KMOD_SHIFT) ) { // '>'
-					unsigned int my_Width,my_Height;
-					getsize_sdl(&my_Width,&my_Height);
-					float step=0.2*my_Height;
-					my_Width+=floor(step*movie_aspect);
-					my_Height+=step;
-					resize_sdl(my_Width, my_Height);
+					XCresize_scale(1);
 				} else if(ev.key.keysym.sym==SDLK_PERIOD) {
+					XCresize_percent(100);
 					resize_sdl(ffctv_width, ffctv_height);
 				} else if(ev.key.keysym.sym== SDLK_COMMA) { // ','
-						unsigned int my_Width,my_Height;
-						getsize_sdl(&my_Width,&my_Height);
-						if( movie_aspect < ((float)my_Width/(float)my_Height) )
-							my_Width=rint((float)my_Height * movie_aspect);
-						else 	my_Height=rint((float)my_Width / movie_aspect);
-						resize_sdl(my_Width, my_Height);
+					XCresize_aspect(0);
 				} else if(ev.key.keysym.sym==SDLK_o) {
-					if (OSD_mode&OSD_OFFF) {
-						OSD_mode&=~OSD_OFFF;
-						OSD_mode|=OSD_OFFS;
-					} else if (OSD_mode&OSD_OFFS) {
-						OSD_mode^=OSD_OFFS;
-					} else {
-						OSD_mode^=OSD_OFFF;
-					}
-					force_redraw=1;
+					ui_osd_offset_cycle();
+				} else if(ev.key.keysym.sym==SDLK_p) {
+					ui_osd_permute();
+				} else if(ev.key.keysym.sym== SDLK_BACKSLASH) {
+					XCtimeoffset(0, (unsigned int) key);
 				} else if(ev.key.keysym.sym== SDLK_EQUALS && ev.key.keysym.mod&KMOD_SHIFT) { // '+' SDLK_PLUS does not work :/
-					if ((interaction_override&OVR_AVOFFSET) != 0 ) {
-						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
-						break;
-					}
-					ts_offset++;
-					force_redraw=1;
-					if (smpte_offset) free(smpte_offset);
-					smpte_offset= calloc(15,sizeof(char));
-					frame_to_smptestring(smpte_offset,ts_offset);
+					XCtimeoffset(1, (unsigned int) key);
 				} else if(ev.key.keysym.sym==SDLK_MINUS) {
-					if ((interaction_override&OVR_AVOFFSET) != 0 ) {
-						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
-						break;
-					}
-					ts_offset--;
-					force_redraw=1;
-					if (smpte_offset) free(smpte_offset);
-					smpte_offset= calloc(15,sizeof(char));
-					frame_to_smptestring(smpte_offset,ts_offset);
+					XCtimeoffset(-1, (unsigned int) key);
 				} else if(ev.key.keysym.sym== SDLK_LEFTBRACKET && ev.key.keysym.mod&KMOD_SHIFT) { // '{'
-					if ((interaction_override&OVR_AVOFFSET) != 0 ) {
-						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
-						break;
-					}
-					if (framerate > 0) {
-						ts_offset-= framerate *60;
-					} else {
-						ts_offset-= 25*60;
-					}
-					force_redraw=1;
-					if (smpte_offset) free(smpte_offset);
-					smpte_offset= calloc(15,sizeof(char));
-					frame_to_smptestring(smpte_offset,ts_offset);
+					XCtimeoffset(-2, (unsigned int) key);
 				} else if(ev.key.keysym.sym== SDLK_RIGHTBRACKET&& ev.key.keysym.mod&KMOD_SHIFT) { // '}'
-					if ((interaction_override&OVR_AVOFFSET) != 0 ) {
-						remote_notify(NTY_KEYBOARD, 310, "keypress=%d", (unsigned int) key);
-						break;
-					}
-					if (framerate > 0) {
-						ts_offset+= framerate *60;
-					} else {
-						ts_offset+= 25*60;
-					}
-					force_redraw=1;
-					if (smpte_offset) free(smpte_offset);
-					smpte_offset= calloc(15,sizeof(char));
-					frame_to_smptestring(smpte_offset,ts_offset);
+					XCtimeoffset(2, (unsigned int) key);
 #ifdef CROPIMG
 				} else if(ev.key.keysym.sym== SDLK_LEFTBRACKET) { // '['
 				} else if(ev.key.keysym.sym== SDLK_RIGHTBRACKET) { // ']'
@@ -469,39 +448,22 @@ void handle_X_events_sdl (void) {
 				sdl_screen = SDL_SetVideoMode(ev.resize.w, ev.resize.h, 0, MYSDLFLAGS);
 				sdl_rect.w=ev.resize.w;
 				sdl_rect.h=ev.resize.h;
-				force_redraw=1;
 				resized_sdl();
+				force_redraw=1;
 				break;
 			case SDL_MOUSEBUTTONUP:
-				if(ev.button.button == SDL_BUTTON_LEFT) {
-					resize_sdl(ffctv_width, ffctv_height);
-				} else 
-#if 0 // fix aspect only on right-button and scroll. 
-					 if(ev.button.button == SDL_BUTTON_WHEELUP ||
-						ev.button.button == SDL_BUTTON_WHEELDOWN ||
-						ev.button.button == SDL_BUTTON_RIGHT)
-#endif
-					 { 
-
-					unsigned int my_Width,my_Height;
-					getsize_sdl(&my_Width,&my_Height);
-
-					if(ev.button.button == SDL_BUTTON_WHEELUP) {
-						float step=sqrt((float)my_Height);
-						my_Width-=floor(step*movie_aspect);
-						my_Height-=step;
-					}
-					if(ev.button.button == SDL_BUTTON_WHEELDOWN) {
-						float step=sqrt((float)my_Height);
-						my_Width+=floor(step*movie_aspect);
-						my_Height+=step;
-					} 
-					// resize to match movie aspect ratio
-					if( movie_aspect < ((float)my_Width/(float)my_Height) )
-						my_Width=floor((float)my_Height * movie_aspect);
-					else my_Height=floor((float)my_Width / movie_aspect);
-
-					resize_sdl(my_Width,my_Height);
+				switch(ev.button.button) {
+					case SDL_BUTTON_WHEELUP:
+						XCresize_aspect(-1);
+						break;
+					case SDL_BUTTON_WHEELDOWN:
+						XCresize_aspect(1);
+						break;
+					case SDL_BUTTON_MIDDLE:
+						XCresize_aspect(0);
+						break;
+					default:
+						break;
 				}
 				break;
       case SDL_ACTIVEEVENT:			/** Application loses/gains visibility */
