@@ -33,6 +33,9 @@ int xoffset = 0;
 #include <X11/xpm.h>
 #include "icons/xjadeo8.xpm"
 
+void xapi_open (void *d);
+void xapi_close (void *d);
+
 extern const  vidout VO[];
 extern int    OSD_mode; // change via keystroke
 
@@ -270,12 +273,21 @@ static void xj_handle_X_events (void) {
 #ifdef XDLG
 		if (handle_xdlg_event(xj_dpy, &event)) continue;
 #endif
-#ifdef DND
-		if (handle_dnd_event(xj_dpy, xj_win, &event)) continue;
+#ifdef XFIB
+		if (x_fib_handle_events (xj_dpy, &event)) {
+			if (x_fib_status () > 0) {
+				char *fn = x_fib_filename ();
+				xapi_open (fn);
+				free (fn);
+			}
+		}
 #endif
 		if (event.xany.window != xj_win) {
 			continue;
 		}
+#ifdef DND
+		if (handle_dnd_event(xj_dpy, xj_win, &event)) continue;
+#endif
 		switch (event.type) {
 			case Expose:
 				xj_render();
@@ -286,6 +298,7 @@ static void xj_handle_X_events (void) {
 				}
 				break;
 			case ConfigureNotify:
+				if (event.xconfigure.width > 1 && event.xconfigure.height > 1)
 				{
 					unsigned int my_Width,my_Height;
 					xj_get_window_size(&my_Width,&my_Height);
@@ -326,9 +339,11 @@ static void xj_handle_X_events (void) {
 				} else
 #ifdef XDLG
 				if (event.xbutton.button == 3) {
-					show_x_dialog(xj_dpy, xj_win,
-							event.xbutton.x_root, event.xbutton.y_root
-							);
+					if (event.xbutton.x >= 0 && event.xbutton.x < xj_dwidth
+							&& event.xbutton.y >= 0 && event.xbutton.y < xj_dheight)
+						show_x_dialog(xj_dpy, xj_win,
+								event.xbutton.x_root, event.xbutton.y_root
+								);
 				} else
 #endif
 				{
@@ -353,10 +368,24 @@ static void xj_handle_X_events (void) {
 					char buf[100];
 					static XComposeStatus stat;
 
-					XLookupString(&event.xkey, buf, sizeof(buf), &key, &stat);
-
-				/* HARDCODED KEY BINDINGS */
-					if        (key == XK_Escape) { // 'Esc'
+					int n = XLookupString(&event.xkey, buf, sizeof(buf), &key, &stat);
+					if (event.xkey.state & ControlMask && n == 1 && key == XK_o) {
+#ifdef XFIB
+						if (!(interaction_override & OVR_LOADFILE)) {
+							x_fib_cfg_filter_callback(fib_filter_movie_filename);
+							x_fib_show (xj_dpy, xj_win, 0, 0);
+						}
+#endif
+					}
+					else if (event.xkey.state & ControlMask && n == 1 && key == XK_w) {
+						if (!(interaction_override & OVR_LOADFILE))
+							xapi_close (NULL);
+					}
+					else if (event.xkey.state & ControlMask && n == 1 && key == XK_q) {
+						if (!(interaction_override&OVR_QUIT_WMG))
+							loop_flag = 0;
+					}
+					else if (key == XK_Escape) { // 'Esc'
 						if ((interaction_override&OVR_QUIT_KEY) == 0) {
 							loop_flag=0;
 						} else {
@@ -390,6 +419,8 @@ static void xj_handle_X_events (void) {
 						ui_osd_fileinfo();
 					} else if (key == XK_g ) { //'g' // Geometry Into
 						ui_osd_geo();
+					} else if (key == XK_r ) { //'r' // OSD - out of range
+						ui_osd_outofrange();
 					} else if (key == XK_x ) { //'x' // OSD - seek position
 						ui_osd_pos();
 					} else if (key == XK_C ) { //'C' // OSD - clear all
@@ -974,6 +1005,9 @@ int open_window_xv (void) {
 void close_window_xv(void) {
 #ifdef XDLG
 	close_x_dialog(xj_dpy);
+#endif
+#ifdef XFIB
+	x_fib_close (xj_dpy);
 #endif
 	XvStopVideo(xj_dpy, xv_port, xj_win);
 	if(xv_shminfo.shmaddr) {
