@@ -32,9 +32,11 @@ static XContext _dlg_ctx = 0;
 static int      _dlg_font_height = 0;
 static int      _dlg_font_ascent = 0;
 static XColor   _c_gray1, _c_gray2;
+static Font     _dlgfont = 0;
 
 static Window   _dlg_mwin = 0;
 static Window   _dlg_swin = 0;
+static Window   _parent = 0;
 
 struct XjxMenuItem {
 	const char *text;
@@ -83,6 +85,14 @@ static void ui_offset_mm()  { XCtimeoffset(-2, 0); }
 static void ui_offset_ph()  { XCtimeoffset( 3, 0); }
 static void ui_offset_mh()  { XCtimeoffset(-3, 0); }
 
+static void ui_quit() {
+	loop_flag=0;
+}
+
+static void ui_open_file() {
+	;
+}
+
 static void ui_close_file() {
 	if (interaction_override&OVR_LOADFILE) return;
 	xapi_close(NULL);
@@ -94,7 +104,7 @@ static void ui_close_file() {
  * - [, ] (pan/crop -- ifdef'ed)
  */
 static struct XjxMenuItem submenu_sync[] = {
-	{"Jack",           "", NULL, &ui_sync_to_jack, 0, 1},
+	{"JACK",           "", NULL, &ui_sync_to_jack, 0, 1},
 	{"LTC",            "", NULL, &ui_sync_to_ltc, 0, 1},
 	{"MTC (JACK)",     "", NULL, &ui_sync_to_mtc_jack, 0, 1},
 	{"MTC (PortMidi)", "", NULL, &ui_sync_to_mtc_portmidi, 0, 1},
@@ -155,25 +165,32 @@ static struct XjxMenuItem submenu_offs[] = {
 };
 
 static struct XjxMenuItem submenu_jack[] = {
-	{"Play/Pause", "<space>",     NULL, &jackt_toggle, 0, 1},
-	{"Play", "",                  NULL, &jackt_start, 0, 1},
-	{"Stop", "",                  NULL, &jackt_stop, 0, 1},
-	{"Rewind",     "<backspace>", NULL, &jackt_rewind, 0, 1},
+	{"Play/Pause", "Space",     NULL, &jackt_toggle, 0, 1},
+	{"Play", "",                NULL, &jackt_start, 0, 1},
+	{"Stop", "",                NULL, &jackt_stop, 0, 1},
+	{"Rewind",     "Backspace", NULL, &jackt_rewind, 0, 1},
 	{NULL, NULL, NULL, NULL, 0, 0},
 };
 
+static struct XjxMenuItem submenu_file[] = {
+	{"Open Video",         "Ctrl+O", NULL, &ui_open_file, 0, 1},
+	{"Close Video",        "Ctrl+W", NULL, &ui_close_file, 0, 1},
+	{"",                   "",       NULL, NULL, 0, 0},
+	{"(Drag&Drop File on", "",       NULL, NULL, 0, 1},
+	{" Window to Load)",   "",       NULL, NULL, 0, 1},
+	{"",                   "",       NULL, NULL, 0, 0},
+	{"Quit",               "Ctrl+Q", NULL, &ui_quit, 0, 1},
+};
+
 static struct XjxMenuItem mainmenu[] = {
 	{"XJadeo " VERSION,    "", NULL, NULL, 0, 1},
 	{"",                   "", NULL, NULL, 0, 0},
+	{"File",               "", submenu_file, NULL, 0, 1},
 	{"Sync",               "", submenu_sync, NULL, 0, 1},
 	{"Display",            "", submenu_size, NULL, 0, 1},
 	{"OSD",                "", submenu_osd,  NULL, 0, 1},
 	{"Offset"   ,          "", submenu_offs, NULL, 0, 1},
 	{"Transport",          "", submenu_jack, NULL, 0, 1},
-	{"",                   "", NULL, NULL, 0, 0},
-	{"Close Video",        "", NULL, &ui_close_file, 0, 1},
-	{"(Drag&Drop File on", "", NULL, NULL, 0, 1},
-	{" Window to Load)",   "", NULL, NULL, 0, 1},
 	{NULL, NULL, NULL, NULL, 0, 0},
 };
 
@@ -269,31 +286,41 @@ static void update_menus () {
 
 	if ((interaction_override&OVR_AVOFFSET) != 0 )
 	{
-		mainmenu[5].sensitive = 0;
+		mainmenu[6].sensitive = 0;
 	} else {
-		mainmenu[5].sensitive = 1;
+		mainmenu[6].sensitive = 1;
 	}
 	if (ui_syncsource() == SYNC_JACK && !(interaction_override&OVR_JCONTROL))
 	{
-		mainmenu[6].sensitive = 1;
+		mainmenu[7].sensitive = 1;
 	} else {
-		mainmenu[6].sensitive = 0;
+		mainmenu[7].sensitive = 0;
 	}
 
 	if (interaction_override & OVR_MENUSYNC) {
-		mainmenu[2].sensitive = 0;
+		mainmenu[3].sensitive = 0;
 	} else {
-		mainmenu[2].sensitive = 1;
+		mainmenu[3].sensitive = 1;
 	}
 
 	if (interaction_override & OVR_LOADFILE) {
-		mainmenu[8].sensitive = 0;
-		mainmenu[9].sensitive = 0;
-		mainmenu[10].sensitive = 0;
+		submenu_file[0].sensitive = 0;
+		submenu_file[1].sensitive = 0;
+		submenu_file[3].sensitive = 0;
+		submenu_file[4].sensitive = 0;
 	} else {
-		mainmenu[8].sensitive = have_open_file() ? 1 : 0;
-		mainmenu[9].sensitive = 1;
-		mainmenu[10].sensitive = 1;
+		submenu_file[0].sensitive = 1;
+		submenu_file[1].sensitive = have_open_file() ? 1 : 0;
+		submenu_file[3].sensitive = 1;
+		submenu_file[4].sensitive = 1;
+	}
+#ifndef XFIB
+	submenu_file[0].sensitive = 0;
+#endif
+	if (interaction_override & OVR_QUIT_WMG) {
+		submenu_file[6].sensitive = 0;
+	} else {
+		submenu_file[6].sensitive = 1;
 	}
 }
 
@@ -327,6 +354,12 @@ static void close_x_dialog_win (Display *dpy, Window *win) {
 	*win = 0;
 }
 
+static uint8_t font_err = 0;
+static int x_error_handler(Display *d, XErrorEvent *e) {
+	font_err = 1;
+	return 0;
+}
+
 static int open_x_dialog_win (
 		Display *dpy, Window *win,
 		int x, int y, int pw,
@@ -346,7 +379,7 @@ static int open_x_dialog_win (
 	attr.event_mask = ExposureMask | KeyPressMask
 		| ButtonPressMask | ButtonReleaseMask
 		| ConfigureNotify | StructureNotifyMask
-		| PointerMotionMask ;
+		| PointerMotionMask | LeaveWindowMask;
 
 	int dlg_width  = 100;
 	int dlg_height = 100;
@@ -361,6 +394,25 @@ static int open_x_dialog_win (
 
 	GC dlg_gc = XCreateGC (dpy, *win, 0, NULL);
 
+	int (*handler)(Display *, XErrorEvent *) = XSetErrorHandler(&x_error_handler);
+#define _XTESTFONT(FN) \
+	{ \
+		font_err = 0; \
+		_dlgfont = XLoadFont(dpy, FN); \
+		XSetFont(dpy, dlg_gc, _dlgfont); \
+		XSync (dpy, False); \
+	}
+
+	font_err = 1;
+	if (getenv("XJFONT")) _XTESTFONT (getenv("XJFONT"));
+	if (font_err) _XTESTFONT ("-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*");
+	if (font_err) _XTESTFONT ("-*-verdana-medium-r-normal-*-12-*-*-*-*-*-*-*");
+	if (font_err) _XTESTFONT ("-misc-fixed-medium-r-normal-*-13-*-*-*-*-*-*-*");
+	if (font_err) _XTESTFONT ("-misc-fixed-medium-r-normal-*-12-*-*-*-*-*-*-*");
+	if (font_err) _dlgfont = None;
+	XSync (dpy, False);
+	XSetErrorHandler(handler);
+
 	if (_dlg_font_height == 0) { // 1st time only
 		if (query_font_geometry (dpy, dlg_gc, "|0Yy", NULL, &_dlg_font_height, &_dlg_font_ascent, NULL)) {
 			XFreeGC (dpy, dlg_gc);
@@ -379,12 +431,13 @@ static int open_x_dialog_win (
 	for(i = 0; i < m_items; ++i) {
 		int cw;
 		if (!query_font_geometry(dpy, dlg_gc, menu[i].text, &cw, NULL, NULL, NULL)) {
+			cw += 4;
 			if (menu[i].key && strlen(menu[i].key) > 0) {
 				int ks = 0;
 				query_font_geometry(dpy, dlg_gc, menu[i].key, &ks, NULL, NULL, NULL);
-				cw += ks + 6;
+				cw += ks + 10;
 			}
-			if (menu[i].submenu) {
+			else if (menu[i].submenu) {
 				cw += 10;
 			}
 			if (cw > max_w) max_w = cw;
@@ -470,9 +523,9 @@ static void dialog_expose (Display *dpy, Window win) {
 		XDrawString (dpy, win, dlg->gc, t_x, t_y, dlg->menu_items[i].text, strlen (dlg->menu_items[i].text));
 		if (dlg->menu_items[i].key && strlen(dlg->menu_items[i].key) > 0) {
 			int ks = 10;
-			if (strlen(dlg->menu_items[i].key) > 1) {
+			if (strlen(dlg->menu_items[i].key) > 0) {
 				query_font_geometry(dpy, dlg->gc, dlg->menu_items[i].key, &ks, NULL, NULL, NULL);
-				ks += 3;
+				ks += 5;
 			}
 			XDrawString (dpy, win, dlg->gc, dlg->width - ks, t_y, dlg->menu_items[i].key, strlen (dlg->menu_items[i].key));
 		}
@@ -545,6 +598,14 @@ static int dialog_click (Display *dpy, Window win, int x, int y, int b) {
 		if (dlg->menu_items[dlg->menu_hover].callback && dlg->menu_items[dlg->menu_hover].sensitive) {
 			dlg->menu_items[dlg->menu_hover].callback();
 			close_x_dialog(dpy);
+#ifdef XFIB
+			if (dlg->menu_items[dlg->menu_hover].callback == &ui_open_file) {
+				if (!(interaction_override&OVR_LOADFILE)) {
+					x_fib_cfg_filter_callback(fib_filter_movie_filename);
+					x_fib_show (dpy, _parent, 0, 0);
+				}
+			}
+#endif
 		}
 	}
 	return 0;
@@ -571,6 +632,7 @@ int show_x_dialog (Display *dpy, Window parent, int x, int y) {
 	if (!_dlg_ctx) {
 		_dlg_ctx = XUniqueContext();
 	}
+	_parent = parent;
 
 	XColor dummy;
 	Colormap colormap = DefaultColormap (dpy, DefaultScreen (dpy));
@@ -608,6 +670,8 @@ void close_x_dialog (Display *dpy) {
 	Colormap colormap = DefaultColormap (dpy, DefaultScreen (dpy));
 	XFreeColors (dpy, colormap, &_c_gray1.pixel, 1, 0);
 	XFreeColors (dpy, colormap, &_c_gray2.pixel, 1, 0);
+	if (_dlgfont != None) XUnloadFont(dpy, _dlgfont);
+	_dlgfont = None;
 	force_redraw = 1;
 }
 
@@ -631,6 +695,19 @@ int handle_xdlg_event (Display *dpy, XEvent *event) {
 		case ConfigureNotify:
 			//printf("DLG ConfigureNotify %dx%d\n", event->xconfigure.width, event->xconfigure.height);
 			break;
+		case LeaveNotify:
+			{
+				struct XJDialog *dlg = NULL;
+				XFindContext (dpy, event->xany.window, _dlg_ctx, (XPointer*)&dlg);
+				if (dlg
+						&& dlg->menu_hover >= 0 && dlg->menu_hover < dlg->menu_count
+						&& !dlg->menu_items[dlg->menu_hover].submenu)
+				{
+					dlg->menu_hover = -1;
+					dialog_expose (dpy, event->xany.window);
+				}
+			}
+			break;
 		case Expose:
 			if (event->xexpose.count == 0) {
 				dialog_expose (dpy, event->xany.window);
